<?php

/* flagscrape.php
 * ``````````````
 * Author: mappu, September 2015
 *
 * Parses and downloads all flag submissions made available on the govt.nz
 *  website as part of the New Zealand 2015 referendum. This script is
 *  deliberately single-threaded to conserve web server resources for those who
 *  might need it more.
 
 * At the time of writing, this scraping activity is NOT PROHIBITED by the
 *  "terms and use" document available on the govt.nz website. HOWEVER, the
 *  scraped data is subject to applicable copyright policies.
 *
 */

class FlagStory {
	public $shortname;
	public $designer_name;
	public $designer_from;
	public $submitter_name;
	public $submitter_from;
	public $text;
	
	public function __construct() {} 
}

class Flag {
	public $image_url;
	public $altdesc;
	public $tags;
	public $official_description;	
	public $stories = [];
	
	public function __construct() {} 
}

function parseHTML($page) {

	if (strpos($page, "You're seeing this page because we can't find the page you are looking for") !== false) {
		return false; // 404
	}
	
	$matches = [];

	$flag = new Flag();

	// Image, Author, Tags

	if (preg_match('~<img src="(/assets/flags-designs/[^"]+)" alt="([^"]+)"~', $page, $matches)) {
		
		$flag->image_url = 'https://www.govt.nz'.$matches[1];
		
		$flag->altdesc = unhesc($matches[2]);
		
		$flag->tags = [];
		
		$tags__ = explode(', tagged with: ', $flag->altdesc);
		if (count($tags__) === 2) {
			$flag->altdesc = $tags__[0];
			$flag->tags = explode(', ', substr($tags__[1], 0, strlen($tags__[1])-1));
		}
		
	}

	// Official Description

	if (preg_match('~<div id="?design-full-description"?[^>]*>(.+?)</div>~ms', $page, $matches)) {
		
		$flag->official_description = unhesc(trim($matches[1]));
	}

	// Blurb

	if (preg_match_all('~<div class="?flag-suggestion-detail"?>(.+?)</div>~ms', $page, $matches, PREG_SET_ORDER)) {	
		foreach($matches as $match) {
			
			$story = new FlagStory();
			$matches_ = [];
			
			if (preg_match('~<h3>(.+?)</h3>~ms', $match[1], $matches_)) {
				$story->shortname = unhesc(trim($matches_[1]));
				
			} elseif (preg_match('~<h1>(.+)?</h1>~ms', $match[1], $matches_)) {
				$story->shortname = str_replace(
					[' (official alternative)', ' (long list)'],
					['', ''],
					strip_tags(unhesc(trim($matches_[1])))
				);
				
			}
			
			if (preg_match('~<p class="?designed-by"?>(?:Designed by: )?(.+?)</p>~ms', $match[1], $matches_)) {
				$rpos = strrpos($matches_[1], ' from ');
				if ($rpos === false) {
					$story->designer_name = unhesc(trim($matches_[1]));
				} else {
					$story->designer_name = unhesc(trim( substr($matches_[1], 0, $rpos)  ));
					$story->designer_from = unhesc(trim( substr($matches_[1], $rpos + 6) ));
				}
			}
			
			if (preg_match('~<p>Suggested by: <span class="?fira-bold"?>(.+?)</span>(?: from (.+?))?</p>~ms', $match[1], $matches_)) {
				$story->submitter_name = unhesc(trim($matches_[1]));
				if (isset($matches_[2])) {
					$story->submitter_from = unhesc(trim($matches_[2]));
				}
			}
			
			if (preg_match('~<p class="?flag-story"?>([^<]+)</p>~ms', $match[1], $matches_)) {
				$story->text = unhesc(trim($matches_[1]));
			}
			
			$flag->stories[] = $story;
			
		}
	}
	
	return $flag;
}

function GET($url) {
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_HEADER, 0);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko');
	curl_setopt($ch, CURLOPT_URL, $url);
	$response = curl_exec($ch);
	curl_close($ch);
	return $response;
}

function unhesc($str) {
	return html_entity_decode($str, ENT_QUOTES | ENT_HTML5, 'UTF-8');
}

function flagscrape($START_ID, $FINISH_ID) {

	for ($i = $START_ID; $i < $FINISH_ID; ++$i) {
		
		echo "\n".$i;
		
		if (file_exists('flags/'.$i.'.json')) {
			echo " (exists, skipping)";
			continue;
		}
		
		for(;;) {
			$page = GET('https://www.govt.nz/browse/engaging-with-government/the-nz-flag-your-chance-to-decide/gallery/design/'.$i);
			if (! is_string($page) || !strlen($page)) {
				echo " (refreshing)";
			} else {
				break;
			}
		}
		
		$flag = parseHTML($page);
		
		if ($flag === false) {
			echo " (bad)";
			continue;
		}
		
		file_put_contents('flags/'.$i.'.json', json_encode($flag, JSON_PRETTY_PRINT));
		
		if ($flag->image_url) {
			$image_body = GET($flag->image_url);
			
			$extension = (strpos($flag->image_url, '.png') === false) ? 'jpg' : 'png';
			file_put_contents('flags/'.$i.'.'.$extension, $image_body);
		}
		
	}

	echo "\n";

}

//
// Entry point
//

error_reporting(E_ALL);
ini_set('display_errors', 'On');

flagscrape( 54 , 32934 );
